import mysql.connector.pooling
import threading
class db_pool:
    event= threading.Event()
    def __init__(self, config, m_pool_size= 10):
        self.__pool= mysql.connector.pooling.MySQLConnectionPool(
            pool_size= m_pool_size,pool_reset_session= True, **config)
            
    def getSession(self):
        try:
            if self.__pool:
                return self.__pool.get_connection()
            else:
                return None
        except Exception as e:
            return None


import queue
from threading import Lock
class Q:
    def __init__(self, maxSize=100):
        self._list = queue.Queue(maxSize)
        self._lock = Lock()

    def push(self, value):
        self._lock.acquire()
        if not self._list.full():
            self._list.put(value)
        self._lock.release()

    def pop(self):
        self._lock.acquire()
        if not self._list.empty():
            return self._list.get()
        self._lock.release()

    def getQSize(self):
        return self._list.qsize()



import logging
import json
import time
from TDhelper.network.http.REST_HTTP import GET
from TDhelper.generic.dynamic.base.Meta import dynamic_creator
from TDhelper.network.rpc.Core.struct import RPC_SERVICE
from TDhelper.Decorators.log import logging,logging_setup
from TDhelper.Decorators.performance import performance
from macpath import split

performance.on_off(True)

class client():
    __TOKEN_KEY__="api-token"
    __TOKEN__=""
    __LOGGER__=logging
    
    @performance.performance_testing
    def __init__(self,uri="",token="",services=None,token_key=None,logger:dict=None):
        if logger:
            self.__LOGGER__.basicConfig(level=logging.INFO)
            self.__LOGGER__.config.dictConfig(logger)
        self.__TOKEN_KEY__= token_key if token_key else self.__TOKEN_KEY__
        self.__TOKEN__=token
        header={
            self.__TOKEN_KEY__:self.__TOKEN__
        }
        if services:
            uri=uri.rstrip("/")
            uri+="?key="+str.split(services,",")
        self.__context__=self.__get_conf__(uri,header)
        if self.__context__:
            self.__dict__["__srv_struct__"]={}
            for k,v in self.__context__.items():
                self.__dict__["__srv_struct__"][k]=type(k,(RPC_SERVICE,),{}).create_by_cnf(v,self.__TOKEN_KEY__)
                methods=[kv for kv in v['methods'].keys()]
                self.__dict__[k]=type(k,(dynamic_creator,),{"__dynamic_methods__":methods,"__hook_method__":self.__call__})()
        else:
            self.__LOGGER__.error("not found rpc method.")
            raise Exception("not found rpc method.")
        super(client,self).__init__()
        
    @performance.performance_testing
    def __call__(self,service_name,fun_name,*args,**kwargs):
        remote_func_name= ("".join([service_name,'.',fun_name]))
        if service_name in self.__srv_struct__:
            if hasattr(self.__srv_struct__[service_name],'methods'):
                if fun_name in self.__srv_struct__[service_name].methods:
                    try:
                        start= time.clock_gettime(0)
                        result= self.__srv_struct__[service_name].__remote_call__(fun_name,*args,**kwargs)
                        end= time.clock_gettime(0)
                        self.__LOGGER__.info("call %s, time consuming %f(s)" % (remote_func_name,(end-start)))
                        return result
                    except Exception as e:
                        self.__LOGGER__.error(e)
                else:
                    self.__LOGGER__.error("in service(%s) not found method(%s)",(service_name,fun_name))
            else:
                self.__LOGGER__.error("service obj not found key(methods).")
        else:
            self.__LOGGER__.error('not found service(%s).'%service_name)
        return {"state": -1, "msg": {}}
        
    def __get_conf__(self,uri,header={}):
        state,body=GET(uri,http_headers=header,time_out=5,charset="utf-8")
        if state==200:
            body=json.loads(body)
            if body["state"]==200:
                return body["msg"]
            else:
                return {}
        return {}


from TDhelper.Msg.InterfaceMsg import *
from TDhelper.apiCore import *

class AppPush(Msg):
    def __init__(self):
        super(AppPush,self).__init__()

    def Send(self, sendTo='', sendBuff='', apiKey='send'):
        if sendTo:
            argsTemplete=self._sendHandle.GetArgsString(apiKey)
            if argsTemplete:
                data= argsTemplete.format(str(sendTo),sendBuff.format(self._sendBy,sendTo,time.localtime()))  #create data
                return self._sendHandle.Call("send", data)  #call remote api

    def MultipleSend(self, sendTo=[], sendBuff='', apiKey='multiplesend'):
        if sendTo:
            argsTemplete=self._sendHandle.GetArgsString(apiKey)
            if argsTemplete:
                data= argsTemplete.format(str(sendTo),sendBuff.format(self._sendBy,sendTo,time.localtime()))  #create data
                return self._sendHandle.Call("send", data)  #call remote api

    def Install(self):
        for item in self._config.Apis:
            self._sendHandle.AddApi(item.Key, item.Uri, item.argsTemplete, item.Type)
        self._sendHandle.SaveToFile()

    def Uninstall(self):
        self._sendHandle.ClearCache()



import array

class Ring:
    def __init__(self, size):
        self._maxSize=size  #Array's max size
        self._readOffset=0  #Array's read offset pos
        self._writeOffset=0 #Array's write offseet pos
        self._ringBuffer=[None]*self._maxSize   #Init array
        self._valueCount=0  #Total this array's value count

    '''
        Get this buffer's size
    '''
    def getSize(self):
        return self._maxSize

    '''
        Total count with value in the buffer
    '''
    def getCount(self):
        return self._valueCount

    '''
        Judge this buffer is full
    '''
    def IsFull(self):
        if self._valueCount== self._maxSize:
            return True
        return False
    
    '''
        Judge this buffer is empty
    '''
    def IsEmpty(self):
        if self._valueCount==0:
            return True
        return False

    '''
        Push value in the array
    '''
    def Push(self,value):
        if self._writeOffset<self._maxSize:
            if self._ringBuffer[self._writeOffset]==None:
                self._ringBuffer[self._writeOffset]=value
                self._valueCount+=1
                self._writeOffset+=1
                if self._writeOffset>=self._maxSize:
                    self._writeOffset=0
                #if write offset equals read offset and the array's index is full after read pos,set write offset is zero
                if self._writeOffset==self._readOffset:
                    if self._readOffset > 0:
                        if self._maxSize - self._readOffset -1  == self._valueCount:
                            self._writeOffset=0
                return True
        return False

    '''
        pop the array's value by offset
    '''
    def Pop(self):
        if self._readOffset<self._maxSize:
            result=self._ringBuffer[self._readOffset]
            if result != None:
                self._ringBuffer[self._readOffset]=None
                self._valueCount-=1
                self._readOffset+=1
                #if array after read offset is full
                if self._maxSize - self._readOffset - 1 == self._valueCount:
                    #if array before read offset is full
                    if self._writeOffset==0 and self._ringBuffer[0] == None:
                        #set write offset is read offset
                        self._writeOffset=self._readOffset
                if self._readOffset>=self._maxSize:
                    self._readOffset=0
                return result
        return False

    '''
        clear the array's value 
    '''
    def Clear(self):
        self._ringBuffer=None
        self._readOffset=0
        self._maxSize=0
        self._writeOffset=0
        self._valueCount=0

    


from copy import deepcopy
import random
from TDhelper.network.http.REST_HTTP import GET
class HostManager:
    def __init__(self) -> None:
        self.__host_cnf_backup__={}
        self.__host_cnf__ = {}
        self.__host__=[]
        self.__host_len__ = 0
        #self.__health_sniffer__ = ""

    def __backup_host__(self):
        self.__host_cnf_backup__= deepcopy(self.__host_cnf__)
        
    def get_per_http_host(self):
        try:
            return self.__generateHost__(self.getHost())
        except Exception as e:
            raise e

    def getHost(self):
        """getHost
            get an access host cnf.
        Parameters:
            self - <class: HostManager>, hostManager instance.
        Returns:
            success - <class, json>, host cnf json.
            fail - None.
        Raises:
            None
        """
        v = 0 if self.__host_len__ == 1 else random.randint(0, self.__host_len__-1)
        result= self.__host_cnf__[self.__host__[v]]
        result["serverId"]= self.__host__[v]
        if result['status']:
            return result
        else:
            return None

    def get_host_by_key(self,key):
        return self.__host_cnf__[key]

    def register(self, serverId, host, port:int=80,sniffer="/api/sniffer", proto:str="http://", status=True):
        """register
            register an rpc host
        Parameters:
            self - <class: HostManager>, hostManager instance.
            serverId - <class: int>, serverId
            host - <class: str>, ip or domain
            port - <class: int>, port. default 80
            sniffer - <class: str>, the healthy sniffer uri. default /api/sniffer
            proto - <class: str>, http:// or https://. default http://
            status - <class: bool>, default True
        Returns:
            None
        Raises:
            10000 - key already exists.
        """
        serverId= str(serverId)
        proto=self.__checken_proto__(proto)
        if serverId not in self.__host_cnf__:
            self.__host_cnf__[serverId]={
                "proto":proto,
                "host":host,
                "port":port,
                "status":status,
                "sniffer":sniffer
            }
            if serverId not in self.__host_cnf_backup__:
                self.__host_cnf_backup__={
                    "proto":proto,
                    "host":host,
                    "port":port,
                    "status":status,
                    "sniffer":sniffer
                }
            self.__host__.append(serverId)
            self.__host_len__= len(self.__host__)
        else:
            raise (Exception("10000, '%s' already exists."%serverId))

    def __checken_proto__(self,v):
        v=v.replace(":","")
        v=v.replace("/","")
        v=v+"://"
        return v

    def reload(self):
        '''reload
            reload host conf by host conf backup.
        '''
        self.__host_cnf__ = {}
        self.__host__=[]
        self.__host_len__ = 0
        #self.__health_sniffer__ = ""
        for k,v in self.__host_cnf_backup__.items():
            self.register(k,v["host"],v["port"],v["sniffer"],v["proto"],v["status"])
    
    def delete(self, serverId):
        """delete
            delete a host cnf.
        Parameters:
            self - <class, HostManager>, hostmanager instance.
            serverId - <class, int|str>, serverId.
        Returns:
            success - serverId
            fail - None
        Raises:
            None.
        """
        if str(serverId) in self.__host_cnf__:
            m_result= self.__host_cnf__.pop(serverId)
            self.__host__.remove(serverId)
            self.__host_len__= len(self.__host__)
            return m_result
        else:
            return None
    
    def health(self):
        """health
            check host health status.
        Parameters:
            self - <class, HostManager>, hostmanager instance.
        Returns:
            <class, json>, cheked result.
            json desc
                statistics - <class, json>, checked result.
                    total - host count.
                    health - checked it's success count.
                    unhealthy - checked it's fail count.
                health - <class, list>, health serverId list.
                unhealthy - <class, list>, unhealthy serverId list.
        Raises:
            None.
        """
        result={
            "statistics":{
                "total":self.__host_len__,
                "health":0,
                "unhealthy":0,
            },
            "health":[],
            "unhealthy":[]
        }
        for k in self.__host__:
            m_uri= self.__generateHost__(self.__host_cnf__[k])
            if m_uri:
                m_uri+= "/"+self.__host_cnf__[k]["sniffer"].strip('/').replace('\\',"/")
                status, body =GET(m_uri,time_out=5)
                if status==200:
                    result["health"].append(k)
                else:
                    result["unhealthy"].append(k)
            else:
                result['unhealthy'].append(k)
        for k in result["unhealthy"]:    
            self.delete(k)
        result["statistics"]["health"]=len(result["health"])
        result["statistics"]["unhealthy"]=len(result["unhealthy"])
        return result

    def __generateHost__(self,uri_cnf:dict):
        proto= uri_cnf["proto"]
        host= uri_cnf["host"]
        port= str(uri_cnf["port"])
        if uri_cnf["status"]:
            if port!=80 or port!="80":
                return proto+host+":"+port
            else:
                return proto+host
        else:
            return None


from platform import python_version
import sys
from types import CodeType, FunctionType

sys.path.append("..")
sys.path.append('/home/tangjing/dev/python/pylib')
from TDhelper.generic.dynamic.base.Meta import Meta,dynamic_creator
#from pylib.TDhelper.generic import dynamic

    

if __name__=="__main__":
    bb=[{'rpc':['ABC','DEFG']},{'bb':['bbc','bbd']}]
    def p(cls_name,name,*args,**kwargs): 
        print("p")
        print(cls_name+"."+name)
        print(args)
        print(kwargs)

    def ec(cls_name,name,*args,**kwargs): 
        print("ec")
        print(cls_name+"."+name)
        print(args)
        print(kwargs)

    cc=type("RPC",(dynamic_creator,),{"__construct__":bb,"__hook_method__":p})()
    #cc.rpc.__set_hook__(ec)
    cc.rpc.ABC(**{"R":"abc method."})
    cc.rpc.DEFG(**{"R":"DEFG method."})


    

    """
    ast create function.
    import ast
    
    code_1=def __method_router__(*args,**kwargs):
        print("__method_router__-------------")
        print(sys._getframe().f_code.co_name)
        print(args)
        print(kwargs)
        print(__name__)"""
    '''
    print(ast.dump(ast.parse(code_1)))
    foo_compile = compile(code_1 , "", "exec")
    foo_code = [ i for i in foo_compile.co_consts][0]
    #[ i for i in foo_compile.co_consts if isinstance(i, CodeType)][0]
    print(foo_code)
    ggg=FunctionType(foo_code,globals())
    ggg()

    from ast import *
    import types
    print(python_version())

    def create_func(func_name):
        function_ast = FunctionDef(
            name=func_name,
            args=arguments(args=[], vararg=None, kwarg=None, defaults=[],kwonlyargs=[],posonlyargs=[],kw_defaults=[]),
            body=[
                Assign(
                    targets=[Name(id='__name__',ctx= Store(),lineno=0,col_offset=0)],
                    value=Str(func_name,lineno=0,col_offset=1),
                    lineno=0,
                    col_offset=0
                ),
                Expr(
                    value=Call(
                        func=Name(id="print",ctx=Load(),lineno=1,col_offset=0),
                        args=[
                            Name(id='__name__',ctx= Load(),lineno=1,col_offset=1)
                        ],
                        keywords=[],
                        lineno=1,
                        col_offset=0
                    ),
                    lineno=1,
                    col_offset=0
                ),
                Return(value=Num(n=42, lineno=2, col_offset=0), lineno=2, col_offset=0,type_comment=None)
            ],
            decorator_list=[],
            type_comment=None,
            lineno=1,
            col_offset=0
        )
        return function_ast

    module_ast = Module(body=[create_func("dynamic_create_func")],type_ignores=[])
    module_code = compile(module_ast, "", "exec")
    function_code = [c for c in module_code.co_consts if isinstance(c, types.CodeType)][0]
    func = types.FunctionType(function_code, globals())
    print(func())
    '''


#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
"""
@File    :   requier.py
@Time    :   2020/04/18 06:48:15
@Author  :   Tang Jing
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
"""

# here put the import lib

# code start

class R:
    def __init__(self, namespace, fromlist=[],loadInstance= True, *args, **kwargs):
        """
        热加载一个类
        参数
         - namespace: string, 格式：generic.requier:R
        """
        self.__userNamespace = namespace
        self.__namespace = None  # namespace
        self.__instance = None  # instance class
        self.__importt(fromlist, loadInstance,*args)  # run import method

    def getNameSpace(self):
        return self.__namespace

    def getInstance(self):
        return self.__instance

    def getType(self):
        if self.__instance:
            return type(self.__instance)
        else:
            raise RuntimeError("instance is null.")

    def __importt(self, fromlist, loadInstance= True, *args):
        """
        Import package
        if import fail will return None
        """
        try:
            if self.__userNamespace:
                local_namespace = str.split(self.__userNamespace, ":")
                if isinstance(local_namespace, list):
                    try:
                        if not fromlist:
                            fromlist.append(local_namespace[1])
                        self.__namespace = __import__(
                            local_namespace[0], fromlist=fromlist
                        )
                    except BaseException as e:
                        raise e
                    if len(local_namespace) == 2 and loadInstance:
                        self.Instance(local_namespace[1], *args)
                else:
                    raise ValueError("namespace(%s) error. namespace:fromlist"%(self.__userNamespace))
            else:
                raise ValueError("TDhelper.generic.requier import 'namespace' is none.")
        except BaseException as e:
            raise e

    def Instance(self, classname, *args):
        """
        Instance class
        If instance fail will return None
        """
        if classname:
            if hasattr(self.__namespace, classname):
                try:
                    self.__instance = getattr(self.__namespace, classname)(*args)
                except Exception as e:
                    raise e
                if self.__instance:
                    return self.__instance
                else:
                    raise RuntimeError("TDhelper.generic.requier(%s) instance %s fail." % (self.__userNamespace,classname))
            else:
                raise RuntimeError("TDhelper.generic.requier(%s) not found %s." % (
                    self.__userNamespace,
                    classname
                ))
        else:
            raise RuntimeError("TDhelper.generic.requier.Instance(%s) classname(%s) is none."%(self.__userNamespace,classname))

    def Call(self, methodname, *args, **kw):
        """
        Call method
        """
        if methodname:
            if self.__instance:
                method = getattr(self.__instance, methodname)
                if method:
                    try:
                        return method(*args, **kw)
                    except Exception as e:
                        raise e
                else:
                    raise RuntimeError("TDhelper.generic.requier.Call '%s' not found in %s." % (
                        methodname,
                        self.__userNamespace,
                    ))
            else:
                raise RuntimeError("TDhelper.generic.requier.Call(%s) instance is none."%(self.__userNamespace))
        else:
            raise RuntimeError("TDhelper.generic.requier.Call(%s) methodname(%s) is none."%(self.__userNamespace,methodname))

    async def call_async(self, methodname, *args, **kw):
        """
        call method by async
        """
        assert methodname, "methodname is none."
        try:
            if methodname:
                if self.__instance:
                    method = getattr(self.__instance, methodname)
                    if method:
                        result = await method(*args, **kw)
                        return True, {"code": 200, "msg": "success", "result": result}
                    else:
                        return False, {
                            "code": 500,
                            "msg": "can not found %s method." % methodname,
                            "result": "",
                        }
                else:
                    return False, {
                        "code": 500,
                        "msg": "instance is none.",
                        "result": "",
                    }
            else:
                return False, {"code": 500, "msg": "methodname is none.", "result": ""}
        except Exception as e:
            return False, {"code": 500, "msg": e, "result": ""}


def InstanceCall(handle, method, *args, **kwargs):
    try:
        method = getattr(handle, method)
        if method:
            return method(*args, **kwargs)
        else:
            raise Exception("%s not have method %s" % (handle, method))
    except Exception as e:
        raise e


if __name__ == "__main__":
    bb = R("a.b:ab").Call("gg")
    print(bb)
    try:
        ccc = R("rest_framework.request:Request",loadInstance=False).getNameSpace().HttpRequest
        print(ccc)
    except Exception as e:
        print(e.args)
    # print(isinstance(1,R(int).getInstance()))



#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
'''
@File    :   script.py
@Time    :   2020/08/27 17:01:29
@Author  :   Tang Jing 
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
'''

# here put the import lib
import threading
from abc import ABCMeta, abstractmethod
from TDhelper.robot.people import actionMode

# code start
class absScript(threading.Thread):
    '''
        动作抽象类，所有动作脚本均继承此类.
    '''
    def __init__(self, handle):
        threading.Thread.__init__(self)
        self.handle= handle # 动作对象句柄
        self.listenStatus= True # 动作监听开关

    def stopListen(self):
        '''
            暂停动作监听
        '''
        self.listenStatus= False

    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    def do(self,mode:actionMode= actionMode.slow):
        '''
            动作执行

            - params
            -   mode: <actionMode>, 动作模式.
        '''
        pass

    @abstractmethod
    def stop(self):
        '''
            暂停动作
        '''
        pass


from TDhelper.robot.control.device import absDevice

class joint:
    '''
        关节基础类, 活动关节控制.
    '''
    def __init__(self, name, powerSystem:dict= None):
        '''
            关节初始化
            - params:
            -   name: <str>, 关节名称.
            -   powerSystem: <dict>, 动力控制模块。{"updown": device,"leftright": device}
        '''
        # self.id= id
        self.name= name
        self.powerSystem= {
            "updown":[None, 0.0], # key:方向指令，arrary[0]:动力设备, arrary[1]:当前值
            "leftright":[None, 0.0]
        }
        if powerSystem:
            if "updown" in powerSystem:
                if isinstance(powerSystem['updown'], absDevice):
                    self.powerSystem['updown'][0]= powerSystem['updown']
                else:
                    raise Exception("%s->updown不是设备." % self.name)
            if "leftright" in powerSystem:
                if isinstance(powerSystem['updown'], absDevice):
                    self.powerSystem["leftright"][0]= powerSystem["leftright"]
                else:
                    raise Exception("%s->leftright不是设备." % self.name)
        else:
            raise Exception("动力控制模块不能为空,至少需要绑定一个动力模块.")
    
    def action(self, actionStruct:dict):
        '''
            控制动作
            - params:
            -   actionStruct: <dict>, 动作指令. 格式：{ "updown": 28, "leftrigth": 40}
        '''
        if actionStruct:
            for action, v in actionStruct.items():
                if self.powerSystem[action.lower()][0]:
                    status =self.powerSystem[action.lower()][0].ctrl(v)
                    if status:
                        self.powerSystem[action.lower][1]= v
                else:
                    raise Exception("%s 动力设备不支持 %s 指令." % (self.name, action.lower()))
        else:
            raise Exception('参数不能为空.')

    def reset(self):
        for k,v in self.powerSystem.items():
            if v[0]:
                v[0].reset()




import json
import logging
import functools
from pickle import TRUE
import re
from rest_framework.response import Response
from rest_framework.request import Request
from types import FunctionType
from TDhelper.generic.classDocCfg import doc
from TDhelper.generic.dictHelper import createDictbyStr, findInDict
from TDhelper.network.http.REST_HTTP import GET, POST, PUT, DELETE, serializePostData, ContentType
from TDhelper.generic.recursion import recursion, recursionCall
from TDhelper.generic.randGetValue import getValue


class register:
    def __init__(self, host, platformKey: str, secret=None, httpHeaders={}):
        self._host = getValue(host).rstrip("/") + "/api/"
        self._platformKey = platformKey
        self._secret = secret
        self._httpHeaders = httpHeaders
        self._httpHeaders["api-token"] = self._secret

    def Register(self, serviceClass:type=None):
        """
        使用方法描述进行注册
        - params:
        -   serviceClass: <class>, 类
        """
        for k, v in serviceClass.__dict__.items():
            if isinstance(v, FunctionType):
                if not k.startswith("__"):
                    if not k.startswith("_"):
                        self._handleRegister(v)

    def RegisterByCfg(self, Cfg: dict):
        """
        使用配置文件进行注册
        - params:
        -   Cfg:<dict>, 注册配置文件.
        """
        if Cfg:
            recursionCall(self._register_permission, 200, Cfg, **{})

    def _handleRegister(self, v):
        k = v.__qualname__.replace(".", "_").upper()
        config = v.__doc__
        config = doc(v.__doc__, "permission")
        if config:
            config = re.sub(r"[\r|\n]", r"", config, count=0, flags=0).strip()
            try:
                config = json.loads(config, encoding="utf-8")
            except:
                config = None
            # todo register permission
            if config:
                recursionCall(self._register_permission, 200, config, **{})
        else:
            raise Exception("config is none.")

    @recursion
    def _register_permission(self, config, **kwargs):
        if self._platformKey:
            config["permission_key"] = (
                self._platformKey + "." + config["permission_key"]
            )
        if not self._host.endswith("/"):
            self._host += "/"
        post_data = {
            "permission_name": config["permission_name"],
            "permission_key": config["permission_key"],
            "permission_domain": config["permission_domain"],
            "permission_uri": config["permission_uri"],
            "permission_enable": config["permission_enable"],
            "permission_parent": config["permission_parent"]
            if "permission_parent" in config
            else kwargs["permission_parent"]
            if "permission_parent" in kwargs
            else 0,
        }
        state, body = POST(
            uri=self._host + "permissions/",
            post_data=post_data,
            http_headers=self._httpHeaders,
            content_type= ContentType.JSON,
            time_out=15,
        )
        m_parent_id = 0
        if state == 200:
            m_ret = str(body, encoding="utf-8")
            m_ret_json = json.loads(m_ret, encoding="utf-8")
            if m_ret_json["state"] == 200:
                m_parent_id = m_ret_json["msg"]["permission_id"]
                logging.info(
                    "create permission '%s' success." % config["permission_name"]
                )
            else:
                logging.info(
                    "create permission '%s' failed.error(%s)"
                    % (config["permission_name"], m_ret_json["msg"])
                )
        else:
            logging.info("create permission '%s' failed." % config["permission_name"])
        if "children" not in config:
            kwargs["break"] = False
        else:
            for item in config["children"]:
                kwargs["permission_parent"] = m_parent_id
                recursionCall(
                    self._register_permission, kwargs["limit"], item, **kwargs
                )
            kwargs["break"] = False
        return config, kwargs


class perACL:
    '''权限检查
    args:
        rpc_key: permission key.
        params_container_class: context class.
        platformKey: platform key.
        tokenKey: token key filed.
    '''
    def __init__(
        self, rpc_key, params_container_class, platformKey=None, tokenKey="usr-token"
    ):
        self._params_container = params_container_class
        self._platformKey = platformKey
        self._tokenKey = tokenKey
        self._rpc = None
        self._rpc_key = rpc_key

    def addRPCHandle(self, handle):
        self._rpc = handle

    def AccessControlLists(self, premissionKey=None, debug=False):
        def decorator(func):
            @functools.wraps(func)
            def wapper(*args, **kwargs):
                validate_state = True
                if not debug:
                    if self._platformKey:
                        self._platformKey += "."
                        self._platformKey = self._platformKey.upper()
                    if premissionKey:
                        _eventKey = self._platformKey + premissionKey.upper()
                    else:
                        _eventKey = (
                            self._platformKey
                            + func.__qualname__.replace(".", "_").upper()
                        )
                    params_instance = None
                    for k in args:
                        if isinstance(k, self._params_container):
                            params_instance = k
                            break
                    if not params_instance:
                        for k, v in kwargs:
                            if isinstance(v, self._params_container):
                                params_instance = v
                                break
                    if not params_instance:
                        return Response("can found context.", status=500)
                    if isinstance(params_instance, Request):
                        if self._tokenKey in params_instance._request.headers:
                            token = params_instance._request.headers[self._tokenKey]
                        else:
                            return Response(
                                "http headers can not found '%s' key." % self._tokenKey,
                                status=500,
                            )
                    elif isinstance(params_instance, dict):
                        token = params_instance[self._tokenKey]
                    elif isinstance(params_instance, (int, str, float)):
                        token = params_instance
                    else:
                        token = getattr(params_instance, self._tokenKey)
                    if token:
                        if self._rpc:
                            validate_ret = self._rpc.call(
                                self._rpc_key, **{"token": token, "event": _eventKey}
                            )
                            if validate_ret:
                                if validate_ret["state"] == 200:
                                    validate_state = True
                                else:
                                    return Response(
                                        "access error.(%s)" % validate_ret["msg"],
                                        status=500,
                                    )
                            else:
                                return Response("access http error.", status=500)
                        else:
                            return Response("rpc handle is none.", status=500)
                    else:
                        return Response("token(%s) is None.", status=500)
                if validate_state:
                    ret = func(*args, **kwargs)
                    return ret
                else:
                    return Response(data="Unauthorized", status=401)

            return wapper
        return decorator
